library ieee;
use ieee.std_logic_1164.all;

package fsi_adc_pkg is
  type t_slv_8x12 is array (7 downto 0) of std_logic_vector(11 downto 0);

  type t_slv_8x12_arr is array (natural range <>) of t_slv_8x12;

  type t_slv_32_arr is array (natural range <>) of std_logic_vector (31 downto 0);
  type t_slv_13_arr is array (natural range <>) of std_logic_vector (12 downto 0);
  type t_slv_9_arr is array (natural range <>) of std_logic_vector (8 downto 0);
  type t_slv_8_arr is array (natural range <>) of std_logic_vector (7 downto 0);
  type t_slv_2_arr is array (natural range <>) of std_logic_vector (1 downto 0);

  --  Commands from system board to fsi boards.
  --  Initial byte to sync the fsi board.
  constant c_sys_syncing   : std_logic_vector(7 downto 0) := b"10100_000";

  --  Idle byte to sync the fsi board, but indicating the system board RX is synced.
  constant c_sys_idle : std_logic_vector(7 downto 0) := b"10100_001";

  --  Request samples
  constant c_sys_sample_on : std_logic_vector(7 downto 0) := b"10100_010";

  --  Stop samples or prbs, back to idle.
  constant c_sys_sample_off : std_logic_vector(7 downto 0) := b"10100_011";

  --  Set value register (followed by 1/4 bytes of data, MSB first)
  constant c_sys_push8  : std_logic_vector(7 downto 0) := b"10100_100";
  constant c_sys_push32 : std_logic_vector(7 downto 0) := b"10100_101";

  --  Execute a command, followed by 1 byte for the command.
  constant c_sys_exec : std_logic_vector(7 downto 0) := b"10100_110";

  --  Commands (from system to periph)
  
  --  Transmit the data buffer.
  constant c_exec_getdata : std_logic_vector(7 downto 0) := x"54";

  --  Set/Get afe_tia_ctrl outputs
  constant c_exec_set_afe : std_logic_vector(7 downto 0) := x"56";
  constant c_exec_get_afe : std_logic_vector(7 downto 0) := x"59";

  --  Send a command to the ADC through SPI.
  --  address: value[31:24], payload: value[15:0]
  constant c_exec_tx_adc : std_logic_vector(7 downto 0) := x"5c";
  --  Get the last received frame
  constant c_exec_rx_adc : std_logic_vector(7 downto 0) := x"55";

  --  Set leds (TODO).
  constant c_exec_leds : std_logic_vector(7 downto 0) := x"51";

  --  Send PRBS
  constant c_exec_prbs : std_logic_vector(7 downto 0) := x"53";

  --  Flash update: (TODO)
  --    Control CS, tx+rx x1, tx 2x4, tx 8x4, rx 2x4, rx 8x4

  --  TODO: Test points, BP i2c, GA.

  --  Commands from fsi board to the system board.

  --  Samples will be send, starting from the next cycle.
  constant c_cmd_samples : std_logic_vector(7 downto 0) := x"12";

  --  Command reply
  constant c_cmd_reply : std_logic_vector(7 downto 0) := x"14";

  --  PRBS will be sent, starting from the next cycle.
  constant c_cmd_prbs : std_logic_vector(7 downto 0) := x"16";
end fsi_adc_pkg;
