library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library UNISIM;
use UNISIM.VComponents.all;

use work.fsi_adc_pkg.all;

entity diot_sys_acq_ip is
  port (
    --  PL clock input
    fpga_pl_clkref_n_i             : std_logic;
    fpga_pl_clkref_p_i             : std_logic;
    
    -- clock outputs:
    m_axi_aclk_100M_o              : out std_logic; -- 100 MHz
    s_axi_hp_fpd_aclk_300M_o       : out std_logic; -- 300 MHz
    m_axi_aresetn_o                : out std_logic;
    pl_resetn_i                    : in  std_logic;

    --  Peripheral board control
    clk_src_sel_o                  : out std_logic_vector(1 downto 0);
    
    -- detect boards
    bckpl_servmod_i                : in std_logic_vector(7 downto 0);

    bp_io_p_i                      : t_slv_13_arr (8 downto 1);
    bp_io_n_i                      : t_slv_13_arr (8 downto 1);
    bp_io_p_o                      : out t_slv_2_arr (8 downto 1);
    bp_io_n_o                      : out t_slv_2_arr (8 downto 1);

    -- access to sys_top_sub_reg
    s_axi_csr_awvalid              : in    std_logic;
    s_axi_csr_awready              : out   std_logic;
    s_axi_csr_awaddr               : in    std_logic_vector(7 downto 2);
--     s_axi_csr_awprot               : in    std_logic_vector(2 downto 0);
    s_axi_csr_wvalid               : in    std_logic;
    s_axi_csr_wready               : out   std_logic;
    s_axi_csr_wdata                : in    std_logic_vector(31 downto 0);
    s_axi_csr_wstrb                : in    std_logic_vector(3 downto 0);
    s_axi_csr_bvalid               : out   std_logic;
    s_axi_csr_bready               : in    std_logic;
    s_axi_csr_bresp                : out   std_logic_vector(1 downto 0);
    s_axi_csr_arvalid              : in    std_logic;
    s_axi_csr_arready              : out   std_logic;
    s_axi_csr_araddr               : in    std_logic_vector(7 downto 2);
--     s_axi_csr_arprot               : in    std_logic_vector(2 downto 0);
    s_axi_csr_rvalid               : out   std_logic;
    s_axi_csr_rready               : in    std_logic;
    s_axi_csr_rdata                : out   std_logic_vector(31 downto 0);
    s_axi_csr_rresp                : out   std_logic_vector(1 downto 0);

    -- DMA 0
    m_axi_dat0_awid                : out std_logic_vector(5 downto 0);
    m_axi_dat0_awaddr              : out std_logic_vector(48 downto 0);
    m_axi_dat0_awlen               : out std_logic_vector(7 downto 0);
    m_axi_dat0_awsize              : out std_logic_vector(2 downto 0);
    m_axi_dat0_awburst             : out std_logic_vector(1 downto 0);
    m_axi_dat0_awprot              : out std_logic_vector(2 downto 0);
    m_axi_dat0_awcache             : out std_logic_vector(3 downto 0);
    m_axi_dat0_awuser              : out std_logic_vector(3 downto 0);
    m_axi_dat0_awvalid             : out std_logic;
    m_axi_dat0_awready             : in std_logic;
    m_axi_dat0_wdata               : out std_logic_vector(127 downto 0);
    m_axi_dat0_wstrb               : out std_logic_vector(15 downto 0);
    m_axi_dat0_wlast               : out std_logic;
    m_axi_dat0_wvalid              : out std_logic;
    m_axi_dat0_wready              : in std_logic;
    m_axi_dat0_bresp               : in std_logic_vector(1 downto 0);
    m_axi_dat0_bvalid              : in std_logic;
    m_axi_dat0_bready              : out std_logic;

    -- DMA 1
    m_axi_dat1_awid                : out std_logic_vector(5 downto 0);
    m_axi_dat1_awaddr              : out std_logic_vector(48 downto 0);
    m_axi_dat1_awlen               : out std_logic_vector(7 downto 0);
    m_axi_dat1_awsize              : out std_logic_vector(2 downto 0);
    m_axi_dat1_awburst             : out std_logic_vector(1 downto 0);
    m_axi_dat1_awprot              : out std_logic_vector(2 downto 0);
    m_axi_dat1_awcache             : out std_logic_vector(3 downto 0);
    m_axi_dat1_awuser              : out std_logic_vector(3 downto 0);
    m_axi_dat1_awvalid             : out std_logic;
    m_axi_dat1_awready             : in std_logic;
    m_axi_dat1_wdata               : out std_logic_vector(127 downto 0);
    m_axi_dat1_wstrb               : out std_logic_vector(15 downto 0);
    m_axi_dat1_wlast               : out std_logic;
    m_axi_dat1_wvalid              : out std_logic;
    m_axi_dat1_wready              : in std_logic;
    m_axi_dat1_bresp               : in std_logic_vector(1 downto 0);
    m_axi_dat1_bvalid              : in std_logic;
    m_axi_dat1_bready              : out std_logic;
    
    led0_o                         : out std_logic;
    led1_o                         : out std_logic
  );
end diot_sys_acq_ip;

architecture arch of diot_sys_acq_ip is
  --  FIXME: is it possible to get rid of this component and use direct
  --  instantiation ?
  component ila_0 IS
    PORT (
      clk : IN STD_LOGIC;
      probe0 : IN STD_LOGIC_VECTOR(63 DOWNTO 0)
    );
  END component ila_0;

  signal clk_100m, clk_300m, clk_400m: std_logic;
  signal clk_100m_oserdes, clk_400m_oserdes : std_logic;
  signal rst_n : std_logic;

  signal pl_reset_n : std_logic;

  signal i2c_board_scl_i : std_logic;
  signal i2c_board_scl_o : std_logic;
  signal i2c_board_scl_t : std_logic;
  signal i2c_board_sda_i : std_logic;
  signal i2c_board_sda_o : std_logic;
  signal i2c_board_sda_t : std_logic;

    --  Reset for DELAY, SERDES and IDELAYCTRL.
  signal io_rst, io_rst_r, io_rst_r2 : std_logic;

  signal status_transfer: std_logic;
  signal control_sw_trig, control_prbs_en : std_logic;
  signal tfr_length : std_logic_vector (31 downto 0);
  signal tfr_address_0, tfr_address_1 : std_logic_vector (31 downto 0);
  signal tfr_cur_address_0, tfr_cur_address_1 : std_logic_vector (31 downto 0) := (others => '1');
  signal tfr_fifo_errs_0, tfr_fifo_errs_1 : std_logic_vector(7 downto 0);
  signal tfr_fifo_max_0, tfr_fifo_max_1 : std_logic_vector(7 downto 0);
  signal tfr_axi_errs_0, tfr_axi_errs_1 : std_logic_vector(3 downto 0);
  signal dma_empty_0, dma_empty_1 : std_logic;
  signal dma_done : std_logic;
  
  signal patterns_sel, patterns_sel_in : std_logic;

  signal samples_patt, samples_periph : t_slv_8x12_arr (7 downto 0);
  signal samples_valid_patt, samples_valid_periph : std_logic;
  signal samples : t_slv_8x12_arr (7 downto 0);
  signal samples_valid : std_logic;

  signal trigger_in0, trigger_in1, trigger_in : std_logic;
  signal periph_triggers, board_off, board_en : std_logic_vector(7 downto 0);
  signal adc_locked, sys_locked, rx_locked, fsi_ready, spi_busy : std_logic_vector(7 downto 0);
  signal periph_ready, periph_trigger, resync, sampling : std_logic;
  signal trig_en : std_logic;

  signal stat_sel_ch    : std_logic_vector(3 downto 0);
  signal stat_sel_board : std_logic_vector(2 downto 0);
  signal stat_sel_delay : std_logic_vector(1 downto 0);

  signal stat_clear : std_logic;
  signal stat_clock : std_logic_vector(63 downto 0);
  signal stat_bit_locked_cnt : std_logic_vector(31 downto 0);
  signal stat_byte_locked_cnt : std_logic_vector(31 downto 0);
  signal stat_cntvalue, stat_prbs_cnt : std_logic_vector(31 downto 0);

  signal delay_inc, delay_dec, delay_vtc : std_logic;
  signal freeze_delays : std_logic;

  signal cmd_boards_sel   : std_logic_vector(7 downto 0);
  signal cmd_value        : std_logic_vector(31 downto 0);
  signal cmd_send         : std_logic;
  signal cmd_exec         : std_logic;
  signal cmd_done         : std_logic_vector(7 downto 0);
  signal cmd_reply_value  : t_slv_32_arr (8 downto 1);
begin
  --  Clocking.
  blk_clock: block
    signal fpga_pl_clkref : std_logic;
    signal clk_fb : std_logic;
    signal clk_buffb : std_logic;
    signal clk_300m_int : std_logic;
    signal clk_400m_int : std_logic;
    signal clk_100m_int : std_logic;
    signal clk_400m_oserdes_int : std_logic;
    signal clk_100m_oserdes_int : std_logic;
    signal clk_locked : std_logic;
    signal pl_reset : std_logic;
    signal rst_cnt : unsigned (3 downto 0);
    signal rst : std_logic;

    --  Ready lines for each idelayctrl (replicated).
    signal dly_rdy : std_logic_vector(0 downto 0);
  begin

    m_axi_aclk_100M_o         <= clk_100m;
    s_axi_hp_fpd_aclk_300M_o  <= clk_300m;
    m_axi_aresetn_o           <= rst_n;
    pl_reset_n                <= pl_resetn_i;

    inst_bufds: IBUFDS
      port map (
        O => fpga_pl_clkref,
        I => fpga_pl_clkref_p_i,
        IB => fpga_pl_clkref_n_i
      );

    inst_MMCME3_BASE : MMCME3_BASE
      generic map (
        BANDWIDTH => "OPTIMIZED", -- Jitter programming (HIGH, LOW, OPTIMIZED)
        CLKFBOUT_MULT_F => 12.0,  -- Multiply value for all CLKOUT (2.000-64.000)
        CLKFBOUT_PHASE => 0.0,    -- Phase offset in degrees of CLKFB (-360.000-360.000)
        CLKIN1_PERIOD => 10.0,    -- Input clock period in ns units, ps resolution (i.e., 33.333 is 30 MHz).
        CLKOUT0_DIVIDE_F => 3.0,  -- Divide amount for CLKOUT0 (1.000-128.000)
        CLKOUT0_DUTY_CYCLE => 0.5,-- CLKOUT0_DUTY_CYCLE - CLKOUT6_DUTY_CYCLE: Duty cycle for each CLKOUT (0.001-0.999).
        CLKOUT1_DUTY_CYCLE => 0.5,
        CLKOUT2_DUTY_CYCLE => 0.5,
        CLKOUT3_DUTY_CYCLE => 0.5,
        CLKOUT4_DUTY_CYCLE => 0.5,
        CLKOUT5_DUTY_CYCLE => 0.5,
        CLKOUT6_DUTY_CYCLE => 0.5,
        CLKOUT0_PHASE => 0.0,-- CLKOUT0_PHASE - CLKOUT6_PHASE: Phase offset for each CLKOUT (-360.000-360.000).
        CLKOUT1_PHASE => 0.0,
        CLKOUT2_PHASE => 0.0,
        CLKOUT3_PHASE => 0.0,
        CLKOUT4_PHASE => 0.0,
        CLKOUT5_PHASE => 0.0,
        CLKOUT6_PHASE => 0.0,
        CLKOUT1_DIVIDE => 4, -- CLKOUT1_DIVIDE - CLKOUT6_DIVIDE: Divide amount for each CLKOUT (1-128)
        CLKOUT2_DIVIDE => 12,
        CLKOUT3_DIVIDE => 3,
        CLKOUT4_DIVIDE => 12,
        CLKOUT5_DIVIDE => 12,
        CLKOUT6_DIVIDE => 12,
        CLKOUT4_CASCADE => "FALSE", -- Cascade CLKOUT4 counter with CLKOUT6 (FALSE, TRUE)
        DIVCLK_DIVIDE => 1,       -- Master division value (1-106)
        IS_CLKFBIN_INVERTED => '0', -- Optional inversion for CLKFBIN
        IS_CLKIN1_INVERTED => '0', -- Optional inversion for CLKIN1
        IS_PWRDWN_INVERTED => '0', -- Optional inversion for PWRDWN
        IS_RST_INVERTED => '0',    -- Optional inversion for RST
        REF_JITTER1 => 0.0,      -- Reference input jitter in UI (0.000-0.999)
        STARTUP_WAIT => "FALSE"  -- Delays DONE until MMCM is locked (FALSE, TRUE)
      )
      port map (
        CLKOUT0 => clk_400m_int,   -- 1-bit output: CLKOUT0
        CLKOUT0B => open,          -- 1-bit output: Inverted CLKOUT0
        CLKOUT1 => clk_300m_int,   -- 1-bit output: CLKOUT1
        CLKOUT1B => open,          -- 1-bit output: Inverted CLKOUT1
        CLKOUT2 => clk_100m_int,   -- 1-bit output: CLKOUT2
        CLKOUT2B => open,          -- 1-bit output: Inverted CLKOUT2
        CLKOUT3 => clk_400m_oserdes_int,        -- 1-bit output: CLKOUT3
        CLKOUT3B => open,          -- 1-bit output: Inverted CLKOUT3
        CLKOUT4 => clk_100m_oserdes_int,        -- 1-bit output: CLKOUT4
        CLKOUT5 => open,           -- 1-bit output: CLKOUT5
        CLKOUT6 => open,           -- 1-bit output: CLKOUT6
        CLKFBOUT => clk_fb,        -- 1-bit output: Feedback clock
        CLKFBOUTB => open,         -- 1-bit output: Inverted CLKFBOUT
        LOCKED => clk_locked,      -- 1-bit output: LOCK
        CLKIN1 => fpga_pl_clkref,  -- 1-bit input: Clock
        PWRDWN => '0',             -- 1-bit input: Power-down
        RST => pl_reset,           -- 1-bit input: Reset
        CLKFBIN => clk_buffb       -- 1-bit input: Feedback clock
      );


    led0_o <= clk_locked;
    pl_reset <= not pl_reset_n;

    --  Status.
    process (clk_100m) is
      variable cnt : natural;
    begin
      if rising_edge (clk_100m) then
        if rst_n = '0' then
          cnt := 0;
          led1_o <= '0';
        else
          if cnt = 50_000_000 - 1 then
            led1_o <= '1';
            cnt := cnt + 1;
          elsif cnt = 100_000_000 - 1 then
            led1_o <= '0';
            cnt := 0;
          else
            cnt := cnt + 1;
          end if;
        end if;
      end if;
    end process;

    inst_bufg_fb: BUFG
      port map (
        I => clk_fb,
        O => clk_buffb
      );

    inst_bufg_clk100m : BUFG
      port map (
        O => clk_100m,         -- 1-bit output: Buffer
        I => clk_100m_int      -- 1-bit input: Buffer
     );

    inst_bufg_300m: BUFG
      port map (
        I => clk_300m_int,
        O => clk_300m
      );

    inst_bufg_clk400m : BUFG
      port map (
        O => clk_400m,         -- 1-bit output: Buffer
        I => clk_400m_int      -- 1-bit input: Buffer
      );

    inst_bufg_clk100m_oserdes : BUFG
      port map (
        O => clk_100m_oserdes,  -- 1-bit output: Buffer
        I => clk_100m_oserdes_int      -- 1-bit input: Buffer
      );

    inst_bufg_clk400m_oserdes : BUFG
     port map (
       O => clk_400m_oserdes, -- 1-bit output: Buffer
       I => clk_400m_oserdes_int      -- 1-bit input: Buffer
    );


    process (pl_reset_n, clk_locked, clk_100m)
    begin
      if pl_reset_n = '0' or clk_locked = '0' then
        rst_n <= '0';
        rst_cnt <= (others => '1');
        io_rst <= '1';
        io_rst_r <= '1';
        io_rst_r2 <= '1';
      elsif rising_edge (clk_100m) then
        io_rst <= '0';
        io_rst_r <= io_rst;
        io_rst_r2 <= io_rst_r;
        --  Note: for iodelay, the reset pulse must be at least 52ns.
        if dly_rdy = (dly_rdy'range => '1') then
          if rst_cnt = 0 then
            rst_n <= '1';
          else
            rst_cnt <= rst_cnt - 1;
          end if;
        end if;
      end if;
    end process;

    gen_idelayctrl: for i in dly_rdy'range generate
      signal idly_rst : std_logic;
      signal idly_rst_400 : std_logic;

      attribute max_fanout : natural;
      attribute max_fanout of idly_rst : signal is 2;
      attribute max_fanout of idly_rst_400 : signal is 2;
    begin
      process (clk_100m)
      begin
        if rising_edge (clk_100m) then
          idly_rst <= io_rst_r;
        end if;
      end process;

      process (clk_400m)
      begin
        if rising_edge (clk_400m) then
          idly_rst_400 <= io_rst;
        end if;
      end process;

      inst_IDELAYCTRL : IDELAYCTRL
        generic map (
          SIM_DEVICE => "ULTRASCALE"
          )
        port map (
          RDY => dly_rdy (i), -- 1-bit output: Ready output
          REFCLK => clk_400m, -- 1-bit input: Reference clock input
          RST => idly_rst_400 -- 1-bit input: Active-High reset input.
          );
    end generate;

    rst <= not rst_n;
  end block;


--  blk_pads: block
--  begin
--    inst_scl_iobuf: component IOBUF
--      port map (
--        I => i2c_board_scl_o,
--        IO => board_scl_b,
--        O => i2c_board_scl_i,
--        T => i2c_board_scl_t
--        );
--    inst_sda_iobuf: component IOBUF
--      port map (
--        I => i2c_board_sda_o,
--        IO => board_sda_b,
--        O => i2c_board_sda_i,
--        T => i2c_board_sda_t
--        );
--    gen_servmod_pads: for i in servmod_b'range generate
--      inst_pad: IOBUF
--      port map (
--        I => '0',
--        IO => servmod_b(i),
--        O => servmod_in(i),
--        T => '1'
--        );
--    end generate;
--  end block;

  --  Registers map
  inst_regs: entity work.sys_top_sub_reg
    port map (
      aclk => clk_100m,
      areset_n => rst_n,
      awvalid => s_axi_csr_awvalid,
      awready => s_axi_csr_awready,
      awaddr => s_axi_csr_awaddr(7 downto 2),
      awprot => (others => '0'),
      wvalid => s_axi_csr_wvalid,
      wready => s_axi_csr_wready,
      wdata => s_axi_csr_wdata,
      wstrb => s_axi_csr_wstrb,
      bvalid => s_axi_csr_bvalid,
      bready => s_axi_csr_bready,
      bresp => s_axi_csr_bresp,
      arvalid => s_axi_csr_arvalid,
      arready => s_axi_csr_arready,
      araddr => s_axi_csr_araddr(7 downto 2),
      arprot => (others => '0'),
      rvalid => s_axi_csr_rvalid,
      rready => s_axi_csr_rready,
      rdata => s_axi_csr_rdata,
      rresp => s_axi_csr_rresp,
      status_dma_done_i => dma_done,
      status_sampling_i => sampling,
      status_transfer_i => status_transfer,
      status_ready_i => periph_ready,
      control_sw_trig_o => control_sw_trig,
      control_prbs_o => control_prbs_en,
      config_periph_off_o => board_off,
      config_clk_sel_o => clk_src_sel_o,
--      config_rst_n_o => rst_n_o,
      config_resync_o => resync,
      config_patterns_o => patterns_sel_in,
      config_trig_en_o => trig_en,
      length_o => tfr_length,
      configs_0_address_o => tfr_address_0,
      configs_0_cur_address_i => tfr_cur_address_0,
      configs_0_status_axi_i => tfr_axi_errs_0,
      configs_0_status_fifo_errs_i => tfr_fifo_errs_0,
      configs_0_status_fifo_max_i => tfr_fifo_max_0,
      configs_1_address_o => tfr_address_1,
      configs_1_cur_address_i => tfr_cur_address_1,
      configs_1_status_axi_i => tfr_axi_errs_1,
      configs_1_status_fifo_errs_i => tfr_fifo_errs_1,
      configs_1_status_fifo_max_i => tfr_fifo_max_1,
      periph_stat1_spi_busy_i => spi_busy,
      periph_stat1_triggers_i => periph_triggers,
      periph_stat1_inp_i => bckpl_servmod_i,
      periph_stat2_fsi_ready_i => fsi_ready,
      periph_stat2_sys_locked_i => sys_locked,
      periph_stat2_adc_locked_i => adc_locked,
      periph_stat2_rx_locked_i => rx_locked,

      cmd_boards_sel_o => cmd_boards_sel,
      cmd_value_o => cmd_value,
      cmd_ctrl_send_o => cmd_send,
      cmd_ctrl_exec_o => cmd_exec,
      cmd_status_done_i => cmd_done,
      cmd_status_done_o => open,
      cmd_status_err_i => '0',
      cmd_status_err_o => open,
      cmd_reply_0_value_i => cmd_reply_value (1),
      cmd_reply_1_value_i => cmd_reply_value (2),
      cmd_reply_2_value_i => cmd_reply_value (3),
      cmd_reply_3_value_i => cmd_reply_value (4),
      cmd_reply_4_value_i => cmd_reply_value (5),
      cmd_reply_5_value_i => cmd_reply_value (6),
      cmd_reply_6_value_i => cmd_reply_value (7),
      cmd_reply_7_value_i => cmd_reply_value (8),

      stat_clock_i => stat_clock,
      stat_sel_ch_o => stat_sel_ch,
      stat_sel_board_o => stat_sel_board,
      stat_sel_delay_o => stat_sel_delay(0),
      stat_sel_branch_o => stat_sel_delay(1),
      stat_sel_clear_o => stat_clear,
      stat_bit_lock_i => stat_bit_locked_cnt,
      stat_byte_lock_i => stat_byte_locked_cnt,
      stat_prbs_i => stat_prbs_cnt,
      delay_cntvalue_i => stat_cntvalue,
      delay_ctrl_inc_o => delay_inc,
      delay_ctrl_dec_o => delay_dec,
      delay_ctrl_freeze_o => freeze_delays,
      delay_ctrl_vtc_en_o => delay_vtc
    );

  --  Samples input
  periph_trigger <= '1' when trig_en = '1' and (board_en and periph_triggers) /= (7 downto 0 => '0') else '0';
  trigger_in0 <= control_sw_trig or periph_trigger;

  process (clk_100m)
  begin
    if rising_edge(clk_100m) then
      if rst_n = '0' then
        trigger_in1 <= '0';
      else
        trigger_in1 <= trigger_in0;
      end if;
    end if;
  end process;

  --  Trigger on a rising edge.
  trigger_in <= trigger_in0 and not trigger_in1;

  inst_patterns: entity work.diot_sys_patterns
    port map (
      clk_100m_i => clk_100m,
      rst_100m_n_i => rst_n,
      samples_o => samples_patt,
      samples_valid_o => samples_valid_patt,
      samples_length_i => tfr_length,
      trigger_i => trigger_in
    );

   inst_core: entity work.sys_fsi_core
    port map (
      clk_100m_i => clk_100m,
      clk_400m_i => clk_400m,
      clk_100m_oserdes_i => clk_100m_oserdes,
      clk_400m_oserdes_i => clk_400m_oserdes,
      rst_n_i => rst_n,
      io_rst_i => io_rst_r2,
      bp_io_p_i => bp_io_p_i,
      bp_io_n_i => bp_io_n_i,
      bp_io_p_o => bp_io_p_o,
      bp_io_n_o => bp_io_n_o,
      resync_i => resync,
      board_en_i => board_en,
      samples_length_i => tfr_length,
      trigger_i => trigger_in,
      prbs_en_i => control_prbs_en,
      triggers_o => periph_triggers,
      samples_o => samples_periph,
      samples_valid_o => samples_valid_periph,
      all_ready_o => periph_ready,
      adc_locked_o => adc_locked,
      sys_locked_o => sys_locked,
      fsi_ready_o => fsi_ready,
      rx_locked_o => rx_locked,
      spi_busy_o => spi_busy,
      sampling_o => sampling,
      stat_sel_board_i => stat_sel_board,
      stat_sel_ch_i => stat_sel_ch,
      stat_sel_dl_i => stat_sel_delay,
      stat_reset_i => stat_clear,
      stat_clock_o => stat_clock,
      stat_bit_locked_cnt_o => stat_bit_locked_cnt,
      stat_byte_locked_cnt_o => stat_byte_locked_cnt,
      stat_prbs_o => stat_prbs_cnt,
      stat_cntvalue_o => stat_cntvalue,
      freeze_delays_i => freeze_delays,
      delay_inc_i => delay_inc,
      delay_dec_i => delay_dec,
      delay_vtc_i => delay_vtc,
      cmd_value_i => cmd_value,
      cmd_board_i => cmd_boards_sel,
      cmd_send_i => cmd_send,
      cmd_exec_i => cmd_exec,
      cmd_ack_o => open,
      cmd_done_o => cmd_done,
      rsp_value_o => cmd_reply_value
    );

  --  To be enabled, a board must be both present and not forced off.
  board_en <= not (board_off or bckpl_servmod_i);

  process (clk_100m, samples_valid)
  begin
    if rising_edge(clk_100m) and samples_valid = '0' then
      patterns_sel <= patterns_sel_in;
    end if;
  end process;

  samples <= samples_patt when patterns_sel = '1' else samples_periph;
  samples_valid <= samples_valid_patt when patterns_sel = '1' else samples_valid_periph;

  status_transfer <= samples_valid;

  --  DMAs
  inst_dma0: entity work.sys_top_dma
    port map (
      clk_100m_i => clk_100m,
      rst_100m_n_i => rst_n,
      clk_300m_i => clk_300m,
      rst_300m_n_i => rst_n,
      samples_i => samples(3 downto 0),
      samples_valid_i => samples_valid,
      samples_start_i => trigger_in,
      dma_start_address_i => tfr_address_0,
      dma_fifo_empty_o => dma_empty_0,
      dma_fifo_max_o => tfr_fifo_max_0,
      dma_fifo_errs_o => tfr_fifo_errs_0,
      dma_axi_errs_o => tfr_axi_errs_0,
      m_axi_dat_awid => m_axi_dat0_awid,
      m_axi_dat_awaddr => m_axi_dat0_awaddr,
      m_axi_dat_awlen => m_axi_dat0_awlen,
      m_axi_dat_awsize => m_axi_dat0_awsize,
      m_axi_dat_awburst => m_axi_dat0_awburst,
      m_axi_dat_awprot => m_axi_dat0_awprot,
      m_axi_dat_awcache => m_axi_dat0_awcache,
      m_axi_dat_awuser => open,
      m_axi_dat_awvalid => m_axi_dat0_awvalid,
      m_axi_dat_awready => m_axi_dat0_awready,
      m_axi_dat_wdata => m_axi_dat0_wdata,
      m_axi_dat_wstrb => m_axi_dat0_wstrb,
      m_axi_dat_wlast => m_axi_dat0_wlast,
      m_axi_dat_wvalid => m_axi_dat0_wvalid,
      m_axi_dat_wready => m_axi_dat0_wready,
      m_axi_dat_bresp => m_axi_dat0_bresp,
      m_axi_dat_bvalid => m_axi_dat0_bvalid,
      m_axi_dat_bready => m_axi_dat0_bready
    );

  inst_dma1: entity work.sys_top_dma
    port map (
      clk_100m_i => clk_100m,
      rst_100m_n_i => rst_n,
      clk_300m_i => clk_300m,
      rst_300m_n_i => rst_n,
      samples_i => samples(7 downto 4),
      samples_valid_i => samples_valid,
      samples_start_i => trigger_in,
      dma_start_address_i => tfr_address_1,
      dma_fifo_empty_o => dma_empty_1,
      dma_fifo_max_o => tfr_fifo_max_1,
      dma_fifo_errs_o => tfr_fifo_errs_1,
      dma_axi_errs_o => tfr_axi_errs_1,
      m_axi_dat_awid => m_axi_dat1_awid,
      m_axi_dat_awaddr => m_axi_dat1_awaddr,
      m_axi_dat_awlen => m_axi_dat1_awlen,
      m_axi_dat_awsize => m_axi_dat1_awsize,
      m_axi_dat_awburst => m_axi_dat1_awburst,
      m_axi_dat_awprot => m_axi_dat1_awprot,
      m_axi_dat_awcache => m_axi_dat1_awcache,
      m_axi_dat_awuser => open,
      m_axi_dat_awvalid => m_axi_dat1_awvalid,
      m_axi_dat_awready => m_axi_dat1_awready,
      m_axi_dat_wdata => m_axi_dat1_wdata,
      m_axi_dat_wstrb => m_axi_dat1_wstrb,
      m_axi_dat_wlast => m_axi_dat1_wlast,
      m_axi_dat_wvalid => m_axi_dat1_wvalid,
      m_axi_dat_wready => m_axi_dat1_wready,
      m_axi_dat_bresp => m_axi_dat1_bresp,
      m_axi_dat_bvalid => m_axi_dat1_bvalid,
      m_axi_dat_bready => m_axi_dat1_bready
    );

process (clk_100m)
  variable armed, dma_done_0, dma_done_1 : boolean;
begin
  if rising_edge(clk_100m) then
    dma_done <= '0';
    if rst_n = '0' then
      armed := false;
    else
      if not armed then
        if sampling = '1' then
          armed := true;
          dma_done_0 := false;
          dma_done_1 := false;
        end if;
      else
        if dma_empty_0 = '1' then
          dma_done_0 := true;
        end if;
        if dma_empty_1 = '1' then
          dma_done_1 := true;
        end if;
        if dma_done_0 and dma_done_1 then
          armed := false;
          dma_done <= '1';
        end if;
      end if;
    end if;
  end if;
end process;

-- inst_ila: ila_0
--   port map (
--       clk => clk_300m,
--       probe0 (0) => m_axi_dat0_awvalid,
--       probe0 (1) => m_axi_dat0_awready,
--       probe0 (2) => m_axi_dat0_wvalid,
--       probe0 (3) => m_axi_dat0_wready,
--       probe0 (4) => m_axi_dat0_bvalid,
--       probe0 (5) => m_axi_dat0_bready,
--       probe0 (7 downto 6) => m_axi_dat0_bresp,
--       probe0 (11 downto 8) => std_logic_vector'(11 downto 8 => '0'), -- dma_address(11 downto 8),
--       probe0 (15 downto 12) => m_axi_dat0_awid (3 downto 0), -- dma_cur_length,
--       probe0 (16) => samples_valid,
--       probe0 (17) => trigger_in,
--       probe0 (18) => rst_n, -- std_logic' ('0'), -- fifo_empty,
--       probe0 (19) => std_logic' ('0'), -- fifo_full,
--       probe0 (20) => std_logic' ('0'), -- dma_running,
--       probe0 (21) => std_logic' ('0'), -- dma_start_burst,
--       probe0 (22) => m_axi_dat0_wlast,
--       probe0 (23) => std_logic' ('0'), -- beat_acked,
--       probe0 (31 downto 24) => std_logic_vector'(31 downto 24 => '0'), -- wr_count,
--       probe0 (63 downto 32) => m_axi_dat0_awaddr(31 downto 0));

--  inst_ila: ila_0
--    port map (
--      clk => clk_100m,
--      probe0 (0) => s_awvalid_csr,
--      probe0 (1) => s_awready_csr,
--      probe0 (9 downto 2) => s_awaddr_csr(7 downto 0),
--      probe0 (13 downto 10) => s_wstrb_csr,
--      probe0 (14) => s_wvalid_csr,
--      probe0 (15) => s_wready_csr,
--      probe0 (23 downto 16) => s_wdata_csr(7 downto 0),
--      probe0 (25 downto 24) => s_bresp_csr,
--      probe0 (26) => s_bvalid_csr,
--      probe0 (27) => s_bready_csr,
--      probe0 (28) => s_arvalid_csr,
--      probe0 (29) => s_arready_csr,
--      probe0 (37 downto 30) => s_araddr_csr(7 downto 0),
--      probe0 (38) => s_rvalid_csr,
--      probe0 (39) => s_rready_csr,
--      probe0 (41 downto 40) => s_rresp_csr,
--      probe0 (49 downto 42) => s_rdata_csr(7 downto 0),
--      probe0 (50) => rst_n,
--      probe0 (63 downto 51) => std_logic_vector'(others => '0'));
end arch;
