library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.fsi_adc_pkg.all;

entity diot_sys_patterns is
  port (
    clk_100m_i : std_logic;
    rst_100m_n_i : std_logic;

    samples_o : out t_slv_8x12_arr (7 downto 0);
    samples_valid_o : out std_logic;

    samples_length_i : std_logic_vector (31 downto 0);
    trigger_i : std_logic
  );
end diot_sys_patterns;

architecture arch of diot_sys_patterns is
  signal samples_count : unsigned(31 downto 0);
  constant c_samples_dly : natural := 4*3; --  Must be a multiple of 3.
  signal samples_dly : natural range 0 to c_samples_dly;
  signal samples_started : std_logic;

  signal samples_valid : std_logic;
begin
  process (clk_100m_i)
    variable patcnt : unsigned (11 downto 0);
    variable sl : unsigned(3 downto 0);
  begin
    if rising_edge(clk_100m_i) then
      if rst_100m_n_i = '0' then
        patcnt := (others => '0');
      else
        if samples_valid = '1' then
          patcnt := patcnt + 1;
        end if;
        for i in samples_o'range loop
          for j in 0 to 7 loop
            sl := to_unsigned(j + 1, 4);
--             samples_o (i)(j) <= std_logic_vector (sl & sl & sl);
            samples_o (i)(j) <= std_logic_vector (x"CA" & sl);
--              (patcnt (7 downto 0) & to_unsigned (j + i*8, 4));
          end loop;
--          samples_o (i)(7) <= std_logic_vector (patcnt);
--          samples_o (i)(6) <=
--            std_logic_vector (to_unsigned (6 + i*8, 8) & "0000");
        end loop;
      end if;
    end if;
  end process;

  samples_valid_o <= samples_valid;

  --  Control pattern generation
  process (clk_100m_i)
  begin
    if rising_edge(clk_100m_i) then
      if rst_100m_n_i = '0' then
        samples_valid <= '0';
        samples_started <= '0';
      else
        if samples_valid = '1'
          or (samples_started = '1' and samples_dly = 0)
        then
          --  When just starting.
          samples_started <= '0';
          samples_valid <= '1';
          --  Sending samples.
          if samples_count = 0 then
            -- End.
            samples_valid <= '0';
          else
            samples_count <= samples_count - 1;
          end if;
        elsif samples_started = '1' then
          --  Triggered but samples are not yet present
          samples_dly <= samples_dly - 1;
        elsif trigger_i = '1' then
          --  Start sending samples
          samples_started <= '1';
          samples_dly <= c_samples_dly - 1;
          samples_count <= unsigned(samples_length_i);
        end if;
      end if;
    end if;
  end process;
end arch;
