library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library UNISIM;
use UNISIM.VComponents.all;

use work.fsi_adc_pkg.all;

entity diot_sys_top is
  port (
    --  PL clock
    fpga_pl_clkref_n_i : std_logic;
    fpga_pl_clkref_p_i : std_logic;

    --  i2c for on board devices (driven by PS)
    board_sda_b : inout std_logic;
    board_scl_b : inout std_logic;

    --  Presence/control of peripheral boards
    servmod_b : inout std_logic_vector(7 downto 0);

    --  Peripheral board control
    rst_n_o : out std_logic;
    clk_src_sel_o : out std_logic_vector(1 downto 0);

    bp_io_p_i : t_slv_13_arr (8 downto 1);
    bp_io_n_i : t_slv_13_arr (8 downto 1);
    bp_io_p_o : out t_slv_2_arr (8 downto 1);
    bp_io_n_o : out t_slv_2_arr (8 downto 1);

    --  Leds (driven by PL)
    led0_o : out std_logic;
    led1_o : out std_logic
  );
end diot_sys_top;

architecture arch of diot_sys_top is
  --  FIXME: is it possible to get rid of this component and use direct
  --  instantiation ?
  component zynq_us is
  port (
    M_AXI_awaddr : out STD_LOGIC_VECTOR ( 39 downto 0 );
    M_AXI_awprot : out STD_LOGIC_VECTOR ( 2 downto 0 );
    M_AXI_awvalid : out STD_LOGIC;
    M_AXI_awready : in STD_LOGIC;
    M_AXI_wdata : out STD_LOGIC_VECTOR ( 31 downto 0 );
    M_AXI_wstrb : out STD_LOGIC_VECTOR ( 3 downto 0 );
    M_AXI_wvalid : out STD_LOGIC;
    M_AXI_wready : in STD_LOGIC;
    M_AXI_bresp : in STD_LOGIC_VECTOR ( 1 downto 0 );
    M_AXI_bvalid : in STD_LOGIC;
    M_AXI_bready : out STD_LOGIC;
    M_AXI_araddr : out STD_LOGIC_VECTOR ( 39 downto 0 );
    M_AXI_arprot : out STD_LOGIC_VECTOR ( 2 downto 0 );
    M_AXI_arvalid : out STD_LOGIC;
    M_AXI_arready : in STD_LOGIC;
    M_AXI_rdata : in STD_LOGIC_VECTOR ( 31 downto 0 );
    M_AXI_rresp : in STD_LOGIC_VECTOR ( 1 downto 0 );
    M_AXI_rvalid : in STD_LOGIC;
    M_AXI_rready : out STD_LOGIC;
    S_AXI_HP0_FPD_aruser : in STD_LOGIC;
    S_AXI_HP0_FPD_awuser : in STD_LOGIC;
    S_AXI_HP0_FPD_awid : in STD_LOGIC_VECTOR ( 5 downto 0 );
    S_AXI_HP0_FPD_awaddr : in STD_LOGIC_VECTOR ( 48 downto 0 );
    S_AXI_HP0_FPD_awlen : in STD_LOGIC_VECTOR ( 7 downto 0 );
    S_AXI_HP0_FPD_awsize : in STD_LOGIC_VECTOR ( 2 downto 0 );
    S_AXI_HP0_FPD_awburst : in STD_LOGIC_VECTOR ( 1 downto 0 );
    S_AXI_HP0_FPD_awlock : in STD_LOGIC;
    S_AXI_HP0_FPD_awcache : in STD_LOGIC_VECTOR ( 3 downto 0 );
    S_AXI_HP0_FPD_awprot : in STD_LOGIC_VECTOR ( 2 downto 0 );
    S_AXI_HP0_FPD_awvalid : in STD_LOGIC;
    S_AXI_HP0_FPD_awready : out STD_LOGIC;
    S_AXI_HP0_FPD_wdata : in STD_LOGIC_VECTOR ( 127 downto 0 );
    S_AXI_HP0_FPD_wstrb : in STD_LOGIC_VECTOR ( 15 downto 0 );
    S_AXI_HP0_FPD_wlast : in STD_LOGIC;
    S_AXI_HP0_FPD_wvalid : in STD_LOGIC;
    S_AXI_HP0_FPD_wready : out STD_LOGIC;
    S_AXI_HP0_FPD_bid : out STD_LOGIC_VECTOR ( 5 downto 0 );
    S_AXI_HP0_FPD_bresp : out STD_LOGIC_VECTOR ( 1 downto 0 );
    S_AXI_HP0_FPD_bvalid : out STD_LOGIC;
    S_AXI_HP0_FPD_bready : in STD_LOGIC;
    S_AXI_HP0_FPD_arid : in STD_LOGIC_VECTOR ( 5 downto 0 );
    S_AXI_HP0_FPD_araddr : in STD_LOGIC_VECTOR ( 48 downto 0 );
    S_AXI_HP0_FPD_arlen : in STD_LOGIC_VECTOR ( 7 downto 0 );
    S_AXI_HP0_FPD_arsize : in STD_LOGIC_VECTOR ( 2 downto 0 );
    S_AXI_HP0_FPD_arburst : in STD_LOGIC_VECTOR ( 1 downto 0 );
    S_AXI_HP0_FPD_arlock : in STD_LOGIC;
    S_AXI_HP0_FPD_arcache : in STD_LOGIC_VECTOR ( 3 downto 0 );
    S_AXI_HP0_FPD_arprot : in STD_LOGIC_VECTOR ( 2 downto 0 );
    S_AXI_HP0_FPD_arvalid : in STD_LOGIC;
    S_AXI_HP0_FPD_arready : out STD_LOGIC;
    S_AXI_HP0_FPD_rid : out STD_LOGIC_VECTOR ( 5 downto 0 );
    S_AXI_HP0_FPD_rdata : out STD_LOGIC_VECTOR ( 127 downto 0 );
    S_AXI_HP0_FPD_rresp : out STD_LOGIC_VECTOR ( 1 downto 0 );
    S_AXI_HP0_FPD_rlast : out STD_LOGIC;
    S_AXI_HP0_FPD_rvalid : out STD_LOGIC;
    S_AXI_HP0_FPD_rready : in STD_LOGIC;
    S_AXI_HP0_FPD_awqos : in STD_LOGIC_VECTOR ( 3 downto 0 );
    S_AXI_HP0_FPD_arqos : in STD_LOGIC_VECTOR ( 3 downto 0 );
    S_AXI_HP3_FPD_aruser : in STD_LOGIC;
    S_AXI_HP3_FPD_awuser : in STD_LOGIC;
    S_AXI_HP3_FPD_awid : in STD_LOGIC_VECTOR ( 5 downto 0 );
    S_AXI_HP3_FPD_awaddr : in STD_LOGIC_VECTOR ( 48 downto 0 );
    S_AXI_HP3_FPD_awlen : in STD_LOGIC_VECTOR ( 7 downto 0 );
    S_AXI_HP3_FPD_awsize : in STD_LOGIC_VECTOR ( 2 downto 0 );
    S_AXI_HP3_FPD_awburst : in STD_LOGIC_VECTOR ( 1 downto 0 );
    S_AXI_HP3_FPD_awlock : in STD_LOGIC;
    S_AXI_HP3_FPD_awcache : in STD_LOGIC_VECTOR ( 3 downto 0 );
    S_AXI_HP3_FPD_awprot : in STD_LOGIC_VECTOR ( 2 downto 0 );
    S_AXI_HP3_FPD_awvalid : in STD_LOGIC;
    S_AXI_HP3_FPD_awready : out STD_LOGIC;
    S_AXI_HP3_FPD_wdata : in STD_LOGIC_VECTOR ( 127 downto 0 );
    S_AXI_HP3_FPD_wstrb : in STD_LOGIC_VECTOR ( 15 downto 0 );
    S_AXI_HP3_FPD_wlast : in STD_LOGIC;
    S_AXI_HP3_FPD_wvalid : in STD_LOGIC;
    S_AXI_HP3_FPD_wready : out STD_LOGIC;
    S_AXI_HP3_FPD_bid : out STD_LOGIC_VECTOR ( 5 downto 0 );
    S_AXI_HP3_FPD_bresp : out STD_LOGIC_VECTOR ( 1 downto 0 );
    S_AXI_HP3_FPD_bvalid : out STD_LOGIC;
    S_AXI_HP3_FPD_bready : in STD_LOGIC;
    S_AXI_HP3_FPD_arid : in STD_LOGIC_VECTOR ( 5 downto 0 );
    S_AXI_HP3_FPD_araddr : in STD_LOGIC_VECTOR ( 48 downto 0 );
    S_AXI_HP3_FPD_arlen : in STD_LOGIC_VECTOR ( 7 downto 0 );
    S_AXI_HP3_FPD_arsize : in STD_LOGIC_VECTOR ( 2 downto 0 );
    S_AXI_HP3_FPD_arburst : in STD_LOGIC_VECTOR ( 1 downto 0 );
    S_AXI_HP3_FPD_arlock : in STD_LOGIC;
    S_AXI_HP3_FPD_arcache : in STD_LOGIC_VECTOR ( 3 downto 0 );
    S_AXI_HP3_FPD_arprot : in STD_LOGIC_VECTOR ( 2 downto 0 );
    S_AXI_HP3_FPD_arvalid : in STD_LOGIC;
    S_AXI_HP3_FPD_arready : out STD_LOGIC;
    S_AXI_HP3_FPD_rid : out STD_LOGIC_VECTOR ( 5 downto 0 );
    S_AXI_HP3_FPD_rdata : out STD_LOGIC_VECTOR ( 127 downto 0 );
    S_AXI_HP3_FPD_rresp : out STD_LOGIC_VECTOR ( 1 downto 0 );
    S_AXI_HP3_FPD_rlast : out STD_LOGIC;
    S_AXI_HP3_FPD_rvalid : out STD_LOGIC;
    S_AXI_HP3_FPD_rready : in STD_LOGIC;
    S_AXI_HP3_FPD_awqos : in STD_LOGIC_VECTOR ( 3 downto 0 );
    S_AXI_HP3_FPD_arqos : in STD_LOGIC_VECTOR ( 3 downto 0 );
    i2c_board_scl_i : in STD_LOGIC;
    i2c_board_scl_o : out STD_LOGIC;
    i2c_board_scl_t : out STD_LOGIC;
    i2c_board_sda_i : in STD_LOGIC;
    i2c_board_sda_o : out STD_LOGIC;
    i2c_board_sda_t : out STD_LOGIC;
    pl_resetn : out STD_LOGIC;
    m_axi_aresetn : in STD_LOGIC;
    m_axi_aclk : in STD_LOGIC;
    s_axi_hp_fpd_aclk : in STD_LOGIC
  );
  end component zynq_us;

  component ila_0 IS
    PORT (
      clk : IN STD_LOGIC;
      probe0 : IN STD_LOGIC_VECTOR(63 DOWNTO 0)
    );
  END component ila_0;

  signal clk_100m, clk_300m, clk_400m: std_logic;
  signal clk_100m_oserdes, clk_400m_oserdes : std_logic_vector(8 downto 1);
  signal rst_n : std_logic;

  signal pl_reset_n : std_logic;
  signal s_awaddr_csr       : std_logic_vector(39 downto 0);
  signal s_awprot_csr       : std_logic_vector(2 downto 0);
  signal s_awvalid_csr      : std_logic;
  signal s_awready_csr      : std_logic;
  signal s_wdata_csr        : std_logic_vector(31 downto 0);
  signal s_wstrb_csr        : std_logic_vector(3 downto 0);
  signal s_wvalid_csr       : std_logic;
  signal s_wready_csr       : std_logic;
  signal s_bresp_csr        : std_logic_vector(1 downto 0);
  signal s_bvalid_csr       : std_logic;
  signal s_bready_csr       : std_logic;
  signal s_araddr_csr       : std_logic_vector(39 downto 0);
  signal s_arprot_csr       : std_logic_vector (2 downto 0);
  signal s_arvalid_csr      : std_logic;
  signal s_arready_csr      : std_logic;
  signal s_rdata_csr        : std_logic_vector(31 downto 0);
  signal s_rresp_csr        : std_logic_vector(1 downto 0);
  signal s_rvalid_csr       : std_logic;
  signal s_rready_csr       : std_logic;

  signal m_axi_dat0_araddr  : std_logic_vector (48 downto 0);
  signal m_axi_dat0_arburst : std_logic_vector (1 downto 0);
  signal m_axi_dat0_arcache : std_logic_vector (3 downto 0);
  signal m_axi_dat0_arid    : std_logic_vector (5 downto 0);
  signal m_axi_dat0_arlen   : std_logic_vector (7 downto 0);
  signal m_axi_dat0_arlock  : std_logic;
  signal m_axi_dat0_arprot  : std_logic_vector (2 downto 0);
  signal m_axi_dat0_arqos   : std_logic_vector (3 downto 0);
  signal m_axi_dat0_arready : std_logic;
  signal m_axi_dat0_arsize  : std_logic_vector (2 downto 0);
  signal m_axi_dat0_aruser  : std_logic;
  signal m_axi_dat0_arvalid : std_logic;
  signal m_axi_dat0_rdata   : std_logic_vector (127 downto 0);
  signal m_axi_dat0_rid     : std_logic_vector (5 downto 0);
  signal m_axi_dat0_rlast   : std_logic;
  signal m_axi_dat0_rready  : std_logic;
  signal m_axi_dat0_rresp   : std_logic_vector (1 downto 0);
  signal m_axi_dat0_rvalid  : std_logic;

  signal m_axi_dat0_awid    : std_logic_vector(5 downto 0);
  signal m_axi_dat0_awaddr  : std_logic_vector(48 downto 0);
  signal m_axi_dat0_awlen   : std_logic_vector(7 downto 0);
  signal m_axi_dat0_awlock  : std_logic := '0';
  signal m_axi_dat0_awsize  : std_logic_vector(2 downto 0);
  signal m_axi_dat0_awburst : std_logic_vector(1 downto 0);
  signal m_axi_dat0_awprot  : std_logic_vector(2 downto 0);
  signal m_axi_dat0_awqos   : std_logic_vector(3 downto 0) := "0000";
  signal m_axi_dat0_awcache : std_logic_vector(3 downto 0);
  signal m_axi_dat0_awuser  : std_logic := '0';
  signal m_axi_dat0_awvalid : std_logic;
  signal m_axi_dat0_awready : std_logic;
  signal m_axi_dat0_wdata   : std_logic_vector(127 downto 0);
  signal m_axi_dat0_wstrb   : std_logic_vector(15 downto 0);
  signal m_axi_dat0_wlast   : std_logic;
  signal m_axi_dat0_wvalid  : std_logic;
  signal m_axi_dat0_wready  : std_logic;
  signal m_axi_dat0_bresp   : std_logic_vector(1 downto 0);
  signal m_axi_dat0_bvalid  : std_logic;
  signal m_axi_dat0_bready  : std_logic;
  signal m_axi_dat0_bid     : std_logic_vector(5 downto 0);

  signal m_axi_dat1_araddr  : std_logic_vector (48 downto 0);
  signal m_axi_dat1_arburst : std_logic_vector (1 downto 0);
  signal m_axi_dat1_arcache : std_logic_vector (3 downto 0);
  signal m_axi_dat1_arid    : std_logic_vector (5 downto 0);
  signal m_axi_dat1_arlen   : std_logic_vector (7 downto 0);
  signal m_axi_dat1_arlock  : std_logic;
  signal m_axi_dat1_arprot  : std_logic_vector (2 downto 0);
  signal m_axi_dat1_arqos   : std_logic_vector (3 downto 0);
  signal m_axi_dat1_arready : std_logic;
  signal m_axi_dat1_arsize  : std_logic_vector (2 downto 0);
  signal m_axi_dat1_aruser  : std_logic;
  signal m_axi_dat1_arvalid : std_logic;
  signal m_axi_dat1_rdata   : std_logic_vector (127 downto 0);
  signal m_axi_dat1_rid     : std_logic_vector (5 downto 0);
  signal m_axi_dat1_rlast   : std_logic;
  signal m_axi_dat1_rready  : std_logic;
  signal m_axi_dat1_rresp   : std_logic_vector (1 downto 0);
  signal m_axi_dat1_rvalid  : std_logic;
  signal m_axi_dat1_bid     : std_logic_vector(5 downto 0);

  signal m_axi_dat1_awid    : std_logic_vector(5 downto 0);
  signal m_axi_dat1_awaddr  : std_logic_vector(48 downto 0);
  signal m_axi_dat1_awlen   : std_logic_vector(7 downto 0);
  signal m_axi_dat1_awlock  : std_logic := '0';
  signal m_axi_dat1_awsize  : std_logic_vector(2 downto 0);
  signal m_axi_dat1_awburst : std_logic_vector(1 downto 0);
  signal m_axi_dat1_awprot  : std_logic_vector(2 downto 0);
  signal m_axi_dat1_awqos   : std_logic_vector(3 downto 0) := "0000";
  signal m_axi_dat1_awcache : std_logic_vector(3 downto 0);
  signal m_axi_dat1_awuser  : std_logic := '0';
  signal m_axi_dat1_awvalid : std_logic;
  signal m_axi_dat1_awready : std_logic;
  signal m_axi_dat1_wdata   : std_logic_vector(127 downto 0);
  signal m_axi_dat1_wstrb   : std_logic_vector(15 downto 0);
  signal m_axi_dat1_wlast   : std_logic;
  signal m_axi_dat1_wvalid  : std_logic;
  signal m_axi_dat1_wready  : std_logic;
  signal m_axi_dat1_bresp   : std_logic_vector(1 downto 0);
  signal m_axi_dat1_bvalid  : std_logic;
  signal m_axi_dat1_bready  : std_logic;

  signal i2c_board_scl_i : std_logic;
  signal i2c_board_scl_o : std_logic;
  signal i2c_board_scl_t : std_logic;
  signal i2c_board_sda_i : std_logic;
  signal i2c_board_sda_o : std_logic;
  signal i2c_board_sda_t : std_logic;

  signal servmod_in : std_logic_vector(7 downto 0);

  --  Reset for DELAY, SERDES and IDELAYCTRL.
  signal io_rst, io_rst_r, io_rst_r2 : std_logic;

  signal status_transfer: std_logic;
  signal control_sw_trig, control_prbs_en : std_logic;
  signal tfr_length : std_logic_vector (31 downto 0);
  signal tfr_address_0, tfr_address_1 : std_logic_vector (31 downto 0);
  signal tfr_cur_address_0, tfr_cur_address_1 : std_logic_vector (31 downto 0) := (others => '1');
  signal tfr_fifo_errs_0, tfr_fifo_errs_1 : std_logic_vector(7 downto 0);
  signal tfr_fifo_max_0, tfr_fifo_max_1 : std_logic_vector(7 downto 0);
  signal tfr_axi_errs_0, tfr_axi_errs_1 : std_logic_vector(3 downto 0);
  signal dma_empty_0, dma_empty_1 : std_logic;
  signal dma_done : std_logic;

  signal patterns_sel, patterns_sel_in : std_logic;

  signal samples_patt, samples_periph : t_slv_8x12_arr (7 downto 0);
  signal samples_valid_patt, samples_valid_periph : std_logic;
  signal samples : t_slv_8x12_arr (7 downto 0);
  signal samples_valid : std_logic;

  signal trigger_in0, trigger_in1, trigger_in : std_logic;
  signal periph_triggers, board_off, board_en : std_logic_vector(7 downto 0);
  signal adc_locked, sys_locked, rx_locked, fsi_ready, spi_busy : std_logic_vector(7 downto 0);
  signal periph_ready, periph_trigger, resync, sampling : std_logic;
  signal trig_en : std_logic;

  signal stat_sel_ch    : std_logic_vector(3 downto 0);
  signal stat_sel_board : std_logic_vector(2 downto 0);
  signal stat_sel_delay : std_logic_vector(1 downto 0);

  signal stat_clear : std_logic;
  signal stat_clock : std_logic_vector(63 downto 0);
  signal stat_bit_locked_cnt : std_logic_vector(31 downto 0);
  signal stat_byte_locked_cnt : std_logic_vector(31 downto 0);
  signal stat_cntvalue, stat_prbs_cnt : std_logic_vector(31 downto 0);

  signal delay_inc, delay_dec, delay_vtc : std_logic;
  signal freeze_delays : std_logic;

  signal cmd_boards_sel   : std_logic_vector(7 downto 0);
  signal cmd_value        : std_logic_vector(31 downto 0);
  signal cmd_send         : std_logic;
  signal cmd_exec         : std_logic;
  signal cmd_done         : std_logic_vector(7 downto 0);
  signal cmd_reply_value  : t_slv_32_arr (8 downto 1);
begin
  --  Clocking.
  blk_clock: block
    signal fpga_pl_clkref : std_logic;
    signal clk_fb : std_logic;
    signal clk_buffb : std_logic;
    signal clk_300m_int : std_logic;
    signal clk_400m_int : std_logic;
    signal clk_locked : std_logic;
    signal pl_reset : std_logic;
    signal rst_cnt : unsigned (3 downto 0);
    signal rst : std_logic;

    signal clk_100m_oserdes_int, clk_400m_oserdes_int : std_logic_vector(3 downto 1);
    signal fpga_pl_clkref_bufg : std_logic;

    --  Ready lines for each idelayctrl (replicated).
    signal dly_rdy : std_logic_vector(41 downto 0);
  begin
    inst_bufds: IBUFDS
      port map (
        O => fpga_pl_clkref,
        I => fpga_pl_clkref_p_i,
        IB => fpga_pl_clkref_n_i
      );

    inst_bufg_fpga_pl : BUFG
      port map (
        O => fpga_pl_clkref_bufg,         -- 1-bit output: Buffer
        I => fpga_pl_clkref      -- 1-bit input: Buffer
     );


    inst_PLLE4_BASE : PLLE4_BASE
      generic map (
        CLKFBOUT_MULT => 12,
        CLKFBOUT_PHASE => 0.0,
        CLKIN_PERIOD => 10.0,
        CLKOUT0_DIVIDE => 3,
        CLKOUT0_DUTY_CYCLE => 0.5,
        CLKOUT0_PHASE => 0.0,
        CLKOUT1_DIVIDE => 4,
        CLKOUT1_DUTY_CYCLE => 0.5,
        CLKOUT1_PHASE => 0.0,
        CLKOUTPHY_MODE => "VCO",
        DIVCLK_DIVIDE => 1,
        IS_CLKFBIN_INVERTED => '0',
        IS_CLKIN_INVERTED => '0',
        IS_PWRDWN_INVERTED => '0',
        IS_RST_INVERTED => '0',
        REF_JITTER => 0.01,
        STARTUP_WAIT => "FALSE"
        )
      port map (
        CLKFBOUT => clk_fb,
        CLKOUT0 => clk_400m_int,
        CLKOUT0B => open,
        CLKOUT1 => clk_300m_int,
        CLKOUT1B => open,
        CLKOUTPHY => open,
        LOCKED => clk_locked,
        CLKFBIN => clk_buffb,
        CLKIN => fpga_pl_clkref_bufg,
        CLKOUTPHYEN => '0',
        PWRDWN => '0',
        RST => pl_reset
      );

    led0_o <= clk_locked;
    pl_reset <= not pl_reset_n;

    --  Status.
    process (clk_100m) is
      variable cnt : natural;
    begin
      if rising_edge (clk_100m) then
        if rst_n = '0' then
          cnt := 0;
          led1_o <= '0';
        else
          if cnt = 50_000_000 - 1 then
            led1_o <= '1';
            cnt := cnt + 1;
          elsif cnt = 100_000_000 - 1 then
            led1_o <= '0';
            cnt := 0;
          else
            cnt := cnt + 1;
          end if;
        end if;
      end if;
    end process;

    inst_bufg_fb: BUFG
      port map (
        I => clk_fb,
        O => clk_buffb
      );

    inst_bufg_clk100m : BUFGCE_DIV
      generic map (
        BUFGCE_DIVIDE => 4,
        IS_CE_INVERTED => '0',  -- Optional inversion for CE
        IS_CLR_INVERTED => '0', -- Optional inversion for CLR
        IS_I_INVERTED => '0'    -- Optional inversion for I
        )
      port map (
        O => clk_100m,         -- 1-bit output: Buffer
        CE => '1',             -- 1-bit input: Buffer enable
        CLR => '0',            -- 1-bit input: Asynchronous clear
        I => clk_400m_int      -- 1-bit input: Buffer
     );

    inst_bufg_300m: BUFG
      port map (
        I => clk_300m_int,
        O => clk_300m
      );

    --  Also use a bufgce_div for clk_400m to reduce (TBC) the skew between
    --  clk_400m and clk_100m
    inst_bufg_clk400m : BUFG
      port map (
        O => clk_400m,         -- 1-bit output: Buffer
        I => clk_400m_int      -- 1-bit input: Buffer
     );

    gen_bufg_oserdes: for i in clk_100m_oserdes_int'range generate
      inst_bufg_clk100m_oserdes : BUFGCE_DIV
        generic map (
          BUFGCE_DIVIDE => 4,
          IS_CE_INVERTED => '0',  -- Optional inversion for CE
          IS_CLR_INVERTED => '0', -- Optional inversion for CLR
          IS_I_INVERTED => '0'    -- Optional inversion for I
          )
        port map (
          O => clk_100m_oserdes_int(i),  -- 1-bit output: Buffer
          CE => '1',             -- 1-bit input: Buffer enable
          CLR => '0',            -- 1-bit input: Asynchronous clear
          I => clk_400m_int      -- 1-bit input: Buffer
       );

      inst_bufgdiv_clk400m_oserdes : BUFG
        port map (
          O => clk_400m_oserdes_int(i), -- 1-bit output: Buffer
          I => clk_400m_int            -- 1-bit input: Buffer
       );
    end generate;

    --  For clock region X2Y3
    clk_100m_oserdes(3 downto 1) <= (3 downto 1 => clk_100m_oserdes_int(1));
    clk_400m_oserdes(3 downto 1) <= (3 downto 1 => clk_400m_oserdes_int(1));
    --  For clock region X2Y5
    clk_100m_oserdes(4) <= clk_100m_oserdes_int(2);
    clk_400m_oserdes(4) <= clk_400m_oserdes_int(2);
    --  For clock region X0Y4
    clk_100m_oserdes(5) <= clk_100m_oserdes_int(2);
    clk_400m_oserdes(5) <= clk_400m_oserdes_int(2);
    --  For clock region X2Y2
    clk_100m_oserdes(6) <= clk_100m_oserdes_int(3);
    clk_400m_oserdes(6) <= clk_400m_oserdes_int(3);
    clk_100m_oserdes(8) <= clk_100m_oserdes_int(3);
    clk_400m_oserdes(8) <= clk_400m_oserdes_int(3);
    --  For clock region X2Y1
    clk_100m_oserdes(7) <= clk_100m_oserdes_int(3);
    clk_400m_oserdes(7) <= clk_400m_oserdes_int(3);

    process (pl_reset_n, clk_locked, clk_100m)
    begin
      if pl_reset_n = '0' or clk_locked = '0' then
        rst_n <= '0';
        rst_cnt <= (others => '1');
        io_rst <= '1';
        io_rst_r <= '1';
        io_rst_r2 <= '1';
      elsif rising_edge (clk_100m) then
        io_rst <= '0';
        io_rst_r <= io_rst;
        io_rst_r2 <= io_rst_r;
        --  Note: for iodelay, the reset pulse must be at least 52ns.
        if dly_rdy = (dly_rdy'range => '1') then
          if rst_cnt = 0 then
            rst_n <= '1';
          else
            rst_cnt <= rst_cnt - 1;
          end if;
        end if;
      end if;
    end process;

    gen_idelayctrl: for i in dly_rdy'range generate
      signal idly_rst : std_logic;
      signal idly_rst_400 : std_logic;

      attribute max_fanout : natural;
      attribute max_fanout of idly_rst : signal is 2;
      attribute max_fanout of idly_rst_400 : signal is 2;
    begin
      process (clk_100m)
      begin
        if rising_edge (clk_100m) then
          idly_rst <= io_rst_r;
        end if;
      end process;

      process (clk_400m)
      begin
        if rising_edge (clk_400m) then
          idly_rst_400 <= io_rst;
        end if;
      end process;

      inst_IDELAYCTRL : IDELAYCTRL
        generic map (
          SIM_DEVICE => "ULTRASCALE"
          )
        port map (
          RDY => dly_rdy (i), -- 1-bit output: Ready output
          REFCLK => clk_400m, -- 1-bit input: Reference clock input
          RST => idly_rst_400 -- 1-bit input: Active-High reset input.
          );
    end generate;

    rst <= not rst_n;
  end block;

  --  Zynq
  inst_zynq_us: zynq_us
    port map (
      --  AXI-4 lite from CPU to registers
      M_AXI_araddr          => s_araddr_csr,
      M_AXI_arprot          => s_arprot_csr,
      M_AXI_arready         => s_arready_csr,
      M_AXI_arvalid         => s_arvalid_csr,
      M_AXI_awaddr          => s_awaddr_csr,
      M_AXI_awprot          => s_awprot_csr,
      M_AXI_awready         => s_awready_csr,
      M_AXI_awvalid         => s_awvalid_csr,
      M_AXI_bready          => s_bready_csr,
      M_AXI_bresp           => s_bresp_csr,
      M_AXI_bvalid          => s_bvalid_csr,
      M_AXI_rdata           => s_rdata_csr,
      M_AXI_rready          => s_rready_csr,
      M_AXI_rresp           => s_rresp_csr,
      M_AXI_rvalid          => s_rvalid_csr,
      M_AXI_wdata           => s_wdata_csr,
      M_AXI_wready          => s_wready_csr,
      M_AXI_wstrb           => s_wstrb_csr,
      M_AXI_wvalid          => s_wvalid_csr,

      --  DMA 0
      S_AXI_HP0_FPD_araddr  => m_axi_dat0_araddr,
      S_AXI_HP0_FPD_arburst => m_axi_dat0_arburst,
      S_AXI_HP0_FPD_arcache => m_axi_dat0_arcache,
      S_AXI_HP0_FPD_arid    => m_axi_dat0_arid,
      S_AXI_HP0_FPD_arlen   => m_axi_dat0_arlen,
      S_AXI_HP0_FPD_arlock  => m_axi_dat0_arlock,
      S_AXI_HP0_FPD_arprot  => m_axi_dat0_arprot,
      S_AXI_HP0_FPD_arqos   => m_axi_dat0_arqos,
      S_AXI_HP0_FPD_arready => m_axi_dat0_arready,
      S_AXI_HP0_FPD_arsize  => m_axi_dat0_arsize,
      S_AXI_HP0_FPD_aruser  => m_axi_dat0_aruser,
      S_AXI_HP0_FPD_arvalid => m_axi_dat0_arvalid,
      S_AXI_HP0_FPD_awaddr  => m_axi_dat0_awaddr,
      S_AXI_HP0_FPD_awburst => m_axi_dat0_awburst,
      S_AXI_HP0_FPD_awcache => m_axi_dat0_awcache,
      S_AXI_HP0_FPD_awid    => m_axi_dat0_awid,
      S_AXI_HP0_FPD_awlen   => m_axi_dat0_awlen,
      S_AXI_HP0_FPD_awlock  => m_axi_dat0_awlock,
      S_AXI_HP0_FPD_awprot  => m_axi_dat0_awprot,
      S_AXI_HP0_FPD_awqos   => m_axi_dat0_awqos,
      S_AXI_HP0_FPD_awready => m_axi_dat0_awready,
      S_AXI_HP0_FPD_awsize  => m_axi_dat0_awsize,
      S_AXI_HP0_FPD_awuser  => m_axi_dat0_awuser,
      S_AXI_HP0_FPD_awvalid => m_axi_dat0_awvalid,
      S_AXI_HP0_FPD_bid     => m_axi_dat0_bid,
      S_AXI_HP0_FPD_bready  => m_axi_dat0_bready,
      S_AXI_HP0_FPD_bresp   => m_axi_dat0_bresp,
      S_AXI_HP0_FPD_bvalid  => m_axi_dat0_bvalid,
      S_AXI_HP0_FPD_rdata   => m_axi_dat0_rdata,
      S_AXI_HP0_FPD_rid     => m_axi_dat0_rid,
      S_AXI_HP0_FPD_rlast   => m_axi_dat0_rlast,
      S_AXI_HP0_FPD_rready  => m_axi_dat0_rready,
      S_AXI_HP0_FPD_rresp   => m_axi_dat0_rresp,
      S_AXI_HP0_FPD_rvalid  => m_axi_dat0_rvalid,
      S_AXI_HP0_FPD_wdata   => m_axi_dat0_wdata,
      S_AXI_HP0_FPD_wlast   => m_axi_dat0_wlast,
      S_AXI_HP0_FPD_wready  => m_axi_dat0_wready,
      S_AXI_HP0_FPD_wstrb   => m_axi_dat0_wstrb,
      S_AXI_HP0_FPD_wvalid  => m_axi_dat0_wvalid,

      --  DMA 1
      S_AXI_HP3_FPD_araddr  => m_axi_dat1_araddr,
      S_AXI_HP3_FPD_arburst => m_axi_dat1_arburst,
      S_AXI_HP3_FPD_arcache => m_axi_dat1_arcache,
      S_AXI_HP3_FPD_arid    => m_axi_dat1_arid,
      S_AXI_HP3_FPD_arlen   => m_axi_dat1_arlen,
      S_AXI_HP3_FPD_arlock  => m_axi_dat1_arlock,
      S_AXI_HP3_FPD_arprot  => m_axi_dat1_arprot,
      S_AXI_HP3_FPD_arqos   => m_axi_dat1_arqos,
      S_AXI_HP3_FPD_arready => m_axi_dat1_arready,
      S_AXI_HP3_FPD_arsize  => m_axi_dat1_arsize,
      S_AXI_HP3_FPD_aruser  => m_axi_dat1_aruser,
      S_AXI_HP3_FPD_arvalid => m_axi_dat1_arvalid,
      S_AXI_HP3_FPD_awaddr  => m_axi_dat1_awaddr,
      S_AXI_HP3_FPD_awburst => m_axi_dat1_awburst,
      S_AXI_HP3_FPD_awcache => m_axi_dat1_awcache,
      S_AXI_HP3_FPD_awid    => m_axi_dat1_awid,
      S_AXI_HP3_FPD_awlen   => m_axi_dat1_awlen,
      S_AXI_HP3_FPD_awlock  => m_axi_dat1_awlock,
      S_AXI_HP3_FPD_awprot  => m_axi_dat1_awprot,
      S_AXI_HP3_FPD_awqos   => m_axi_dat1_awqos,
      S_AXI_HP3_FPD_awready => m_axi_dat1_awready,
      S_AXI_HP3_FPD_awsize  => m_axi_dat1_awsize,
      S_AXI_HP3_FPD_awuser  => m_axi_dat1_awuser,
      S_AXI_HP3_FPD_awvalid => m_axi_dat1_awvalid,
      S_AXI_HP3_FPD_bid     => m_axi_dat1_bid,
      S_AXI_HP3_FPD_bready  => m_axi_dat1_bready,
      S_AXI_HP3_FPD_bresp   => m_axi_dat1_bresp,
      S_AXI_HP3_FPD_bvalid  => m_axi_dat1_bvalid,
      S_AXI_HP3_FPD_rdata   => m_axi_dat1_rdata,
      S_AXI_HP3_FPD_rid     => m_axi_dat1_rid,
      S_AXI_HP3_FPD_rlast   => m_axi_dat1_rlast,
      S_AXI_HP3_FPD_rready  => m_axi_dat1_rready,
      S_AXI_HP3_FPD_rresp   => m_axi_dat1_rresp,
      S_AXI_HP3_FPD_rvalid  => m_axi_dat1_rvalid,
      S_AXI_HP3_FPD_wdata   => m_axi_dat1_wdata,
      S_AXI_HP3_FPD_wlast   => m_axi_dat1_wlast,
      S_AXI_HP3_FPD_wready  => m_axi_dat1_wready,
      S_AXI_HP3_FPD_wstrb   => m_axi_dat1_wstrb,
      S_AXI_HP3_FPD_wvalid  => m_axi_dat1_wvalid,

      i2c_board_scl_i       => i2c_board_scl_i,
      i2c_board_scl_o       => i2c_board_scl_o,
      i2c_board_scl_t       => i2c_board_scl_t,
      i2c_board_sda_i       => i2c_board_sda_i,
      i2c_board_sda_o       => i2c_board_sda_o,
      i2c_board_sda_t       => i2c_board_sda_t,

      m_axi_aclk            => clk_100m,
      m_axi_aresetn         => rst_n,
      pl_resetn             => pl_reset_n,
      s_axi_hp_fpd_aclk     => clk_300m
    );

  --  Read port is not used.
  m_axi_dat0_arvalid <= '0';
  m_axi_dat1_arvalid <= '0';
  m_axi_dat0_rready <= '0';
  m_axi_dat1_rready <= '0';

  blk_pads: block
  begin
    inst_scl_iobuf: component IOBUF
      port map (
        I => i2c_board_scl_o,
        IO => board_scl_b,
        O => i2c_board_scl_i,
        T => i2c_board_scl_t
        );
    inst_sda_iobuf: component IOBUF
      port map (
        I => i2c_board_sda_o,
        IO => board_sda_b,
        O => i2c_board_sda_i,
        T => i2c_board_sda_t
        );
    gen_servmod_pads: for i in servmod_b'range generate
      inst_pad: IOBUF
      port map (
        I => '0',
        IO => servmod_b(i),
        O => servmod_in(i),
        T => '1'
        );
    end generate;
  end block;

  --  Registers map
  inst_regs: entity work.sys_top_sub_reg
    port map (
      aclk => clk_100m,
      areset_n => rst_n,
      awvalid => s_awvalid_csr,
      awready => s_awready_csr,
      awaddr => s_awaddr_csr(7 downto 2),
      awprot => (others => '0'),
      wvalid => s_wvalid_csr,
      wready => s_wready_csr,
      wdata => s_wdata_csr,
      wstrb => s_wstrb_csr,
      bvalid => s_bvalid_csr,
      bready => s_bready_csr,
      bresp => s_bresp_csr,
      arvalid => s_arvalid_csr,
      arready => s_arready_csr,
      araddr => s_araddr_csr(7 downto 2),
      arprot => (others => '0'),
      rvalid => s_rvalid_csr,
      rready => s_rready_csr,
      rdata => s_rdata_csr,
      rresp => s_rresp_csr,
      status_dma_done_i => dma_done,
      status_sampling_i => sampling,
      status_transfer_i => status_transfer,
      status_ready_i => periph_ready,
      control_sw_trig_o => control_sw_trig,
      control_prbs_o => control_prbs_en,
      config_periph_off_o => board_off,
      config_clk_sel_o => clk_src_sel_o,
      config_rst_n_o => rst_n_o,
      config_resync_o => resync,
      config_patterns_o => patterns_sel_in,
      config_trig_en_o => trig_en,
      length_o => tfr_length,
      configs_0_address_o => tfr_address_0,
      configs_0_cur_address_i => tfr_cur_address_0,
      configs_0_status_axi_i => tfr_axi_errs_0,
      configs_0_status_fifo_errs_i => tfr_fifo_errs_0,
      configs_0_status_fifo_max_i => tfr_fifo_max_0,
      configs_1_address_o => tfr_address_1,
      configs_1_cur_address_i => tfr_cur_address_1,
      configs_1_status_axi_i => tfr_axi_errs_1,
      configs_1_status_fifo_errs_i => tfr_fifo_errs_1,
      configs_1_status_fifo_max_i => tfr_fifo_max_1,
      periph_stat1_spi_busy_i => spi_busy,
      periph_stat1_triggers_i => periph_triggers,
      periph_stat1_inp_i => servmod_in,
      periph_stat2_fsi_ready_i => fsi_ready,
      periph_stat2_sys_locked_i => sys_locked,
      periph_stat2_adc_locked_i => adc_locked,
      periph_stat2_rx_locked_i => rx_locked,

      cmd_boards_sel_o => cmd_boards_sel,
      cmd_value_o => cmd_value,
      cmd_ctrl_send_o => cmd_send,
      cmd_ctrl_exec_o => cmd_exec,
      cmd_status_done_i => cmd_done,
      cmd_status_done_o => open,
      cmd_status_err_i => '0',
      cmd_status_err_o => open,
      cmd_reply_0_value_i => cmd_reply_value (1),
      cmd_reply_1_value_i => cmd_reply_value (2),
      cmd_reply_2_value_i => cmd_reply_value (3),
      cmd_reply_3_value_i => cmd_reply_value (4),
      cmd_reply_4_value_i => cmd_reply_value (5),
      cmd_reply_5_value_i => cmd_reply_value (6),
      cmd_reply_6_value_i => cmd_reply_value (7),
      cmd_reply_7_value_i => cmd_reply_value (8),

      stat_clock_i => stat_clock,
      stat_sel_ch_o => stat_sel_ch,
      stat_sel_board_o => stat_sel_board,
      stat_sel_delay_o => stat_sel_delay(0),
      stat_sel_branch_o => stat_sel_delay(1),
      stat_sel_clear_o => stat_clear,
      stat_bit_lock_i => stat_bit_locked_cnt,
      stat_byte_lock_i => stat_byte_locked_cnt,
      stat_prbs_i => stat_prbs_cnt,
      delay_cntvalue_i => stat_cntvalue,
      delay_ctrl_inc_o => delay_inc,
      delay_ctrl_dec_o => delay_dec,
      delay_ctrl_freeze_o => freeze_delays,
      delay_ctrl_vtc_en_o => delay_vtc
    );

  --  Samples input
  periph_trigger <= '1' when trig_en = '1' and (board_en and periph_triggers) /= (7 downto 0 => '0') else '0';
  trigger_in0 <= control_sw_trig or periph_trigger;

  process (clk_100m)
  begin
    if rising_edge(clk_100m) then
      if rst_n = '0' then
        trigger_in1 <= '0';
      else
        trigger_in1 <= trigger_in0;
      end if;
    end if;
  end process;

  --  Trigger on a rising edge.
  trigger_in <= trigger_in0 and not trigger_in1;

  inst_patterns: entity work.diot_sys_patterns
    port map (
      clk_100m_i => clk_100m,
      rst_100m_n_i => rst_n,
      samples_o => samples_patt,
      samples_valid_o => samples_valid_patt,
      samples_length_i => tfr_length,
      trigger_i => trigger_in
    );

  inst_core: entity work.sys_fsi_core
    port map (
      clk_100m_i => clk_100m,
      clk_400m_i => clk_400m,
      clk_100m_oserdes_i => clk_100m_oserdes,
      clk_400m_oserdes_i => clk_400m_oserdes,
      rst_n_i => rst_n,
      io_rst_i => io_rst_r2,
      bp_io_p_i => bp_io_p_i,
      bp_io_n_i => bp_io_n_i,
      bp_io_p_o => bp_io_p_o,
      bp_io_n_o => bp_io_n_o,
      resync_i => resync,
      board_en_i => board_en,
      samples_length_i => tfr_length,
      trigger_i => trigger_in,
      prbs_en_i => control_prbs_en,
      triggers_o => periph_triggers,
      samples_o => samples_periph,
      samples_valid_o => samples_valid_periph,
      all_ready_o => periph_ready,
      adc_locked_o => adc_locked,
      sys_locked_o => sys_locked,
      fsi_ready_o => fsi_ready,
      rx_locked_o => rx_locked,
      spi_busy_o => spi_busy,
      sampling_o => sampling,
      stat_sel_board_i => stat_sel_board,
      stat_sel_ch_i => stat_sel_ch,
      stat_sel_dl_i => stat_sel_delay,
      stat_reset_i => stat_clear,
      stat_clock_o => stat_clock,
      stat_bit_locked_cnt_o => stat_bit_locked_cnt,
      stat_byte_locked_cnt_o => stat_byte_locked_cnt,
      stat_prbs_o => stat_prbs_cnt,
      stat_cntvalue_o => stat_cntvalue,
      freeze_delays_i => freeze_delays,
      delay_inc_i => delay_inc,
      delay_dec_i => delay_dec,
      delay_vtc_i => delay_vtc,
      cmd_value_i => cmd_value,
      cmd_board_i => cmd_boards_sel,
      cmd_send_i => cmd_send,
      cmd_exec_i => cmd_exec,
      cmd_ack_o => open,
      cmd_done_o => cmd_done,
      rsp_value_o => cmd_reply_value
    );

  --  To be enabled, a board must be both present and not forced off.
  board_en <= not (board_off or servmod_in);

  process (clk_100m, samples_valid)
  begin
    if rising_edge(clk_100m) and samples_valid = '0' then
      patterns_sel <= patterns_sel_in;
    end if;
  end process;

  samples <= samples_patt when patterns_sel = '1' else samples_periph;
  samples_valid <= samples_valid_patt when patterns_sel = '1' else samples_valid_periph;

  status_transfer <= samples_valid;

  --  DMAs
  inst_dma0: entity work.sys_top_dma
    port map (
      clk_100m_i => clk_100m,
      rst_100m_n_i => rst_n,
      clk_300m_i => clk_300m,
      rst_300m_n_i => rst_n,
      samples_i => samples(3 downto 0),
      samples_valid_i => samples_valid,
      samples_start_i => trigger_in,
      dma_start_address_i => tfr_address_0,
      dma_fifo_empty_o => dma_empty_0,
      dma_fifo_max_o => tfr_fifo_max_0,
      dma_fifo_errs_o => tfr_fifo_errs_0,
      dma_axi_errs_o => tfr_axi_errs_0,
      m_axi_dat_awid => m_axi_dat0_awid,
      m_axi_dat_awaddr => m_axi_dat0_awaddr,
      m_axi_dat_awlen => m_axi_dat0_awlen,
      m_axi_dat_awsize => m_axi_dat0_awsize,
      m_axi_dat_awburst => m_axi_dat0_awburst,
      m_axi_dat_awprot => m_axi_dat0_awprot,
      m_axi_dat_awcache => m_axi_dat0_awcache,
      m_axi_dat_awuser => open,
      m_axi_dat_awvalid => m_axi_dat0_awvalid,
      m_axi_dat_awready => m_axi_dat0_awready,
      m_axi_dat_wdata => m_axi_dat0_wdata,
      m_axi_dat_wstrb => m_axi_dat0_wstrb,
      m_axi_dat_wlast => m_axi_dat0_wlast,
      m_axi_dat_wvalid => m_axi_dat0_wvalid,
      m_axi_dat_wready => m_axi_dat0_wready,
      m_axi_dat_bresp => m_axi_dat0_bresp,
      m_axi_dat_bvalid => m_axi_dat0_bvalid,
      m_axi_dat_bready => m_axi_dat0_bready
    );

  inst_dma1: entity work.sys_top_dma
    port map (
      clk_100m_i => clk_100m,
      rst_100m_n_i => rst_n,
      clk_300m_i => clk_300m,
      rst_300m_n_i => rst_n,
      samples_i => samples(7 downto 4),
      samples_valid_i => samples_valid,
      samples_start_i => trigger_in,
      dma_start_address_i => tfr_address_1,
      dma_fifo_empty_o => dma_empty_1,
      dma_fifo_max_o => tfr_fifo_max_1,
      dma_fifo_errs_o => tfr_fifo_errs_1,
      dma_axi_errs_o => tfr_axi_errs_1,
      m_axi_dat_awid => m_axi_dat1_awid,
      m_axi_dat_awaddr => m_axi_dat1_awaddr,
      m_axi_dat_awlen => m_axi_dat1_awlen,
      m_axi_dat_awsize => m_axi_dat1_awsize,
      m_axi_dat_awburst => m_axi_dat1_awburst,
      m_axi_dat_awprot => m_axi_dat1_awprot,
      m_axi_dat_awcache => m_axi_dat1_awcache,
      m_axi_dat_awuser => open,
      m_axi_dat_awvalid => m_axi_dat1_awvalid,
      m_axi_dat_awready => m_axi_dat1_awready,
      m_axi_dat_wdata => m_axi_dat1_wdata,
      m_axi_dat_wstrb => m_axi_dat1_wstrb,
      m_axi_dat_wlast => m_axi_dat1_wlast,
      m_axi_dat_wvalid => m_axi_dat1_wvalid,
      m_axi_dat_wready => m_axi_dat1_wready,
      m_axi_dat_bresp => m_axi_dat1_bresp,
      m_axi_dat_bvalid => m_axi_dat1_bvalid,
      m_axi_dat_bready => m_axi_dat1_bready
    );

process (clk_100m)
  variable armed, dma_done_0, dma_done_1 : boolean;
begin
  if rising_edge(clk_100m) then
    dma_done <= '0';
    if rst_n = '0' then
      armed := false;
    else
      if not armed then
        if sampling = '1' then
          armed := true;
          dma_done_0 := false;
          dma_done_1 := false;
        end if;
      else
        if dma_empty_0 = '1' then
          dma_done_0 := true;
        end if;
        if dma_empty_1 = '1' then
          dma_done_1 := true;
        end if;
        if dma_done_0 and dma_done_1 then
          armed := false;
          dma_done <= '1';
        end if;
      end if;
    end if;
  end if;
end process;

-- inst_ila: ila_0
--   port map (
--       clk => clk_300m,
--       probe0 (0) => m_axi_dat0_awvalid,
--       probe0 (1) => m_axi_dat0_awready,
--       probe0 (2) => m_axi_dat0_wvalid,
--       probe0 (3) => m_axi_dat0_wready,
--       probe0 (4) => m_axi_dat0_bvalid,
--       probe0 (5) => m_axi_dat0_bready,
--       probe0 (7 downto 6) => m_axi_dat0_bresp,
--       probe0 (11 downto 8) => std_logic_vector'(11 downto 8 => '0'), -- dma_address(11 downto 8),
--       probe0 (15 downto 12) => m_axi_dat0_awid (3 downto 0), -- dma_cur_length,
--       probe0 (16) => samples_valid,
--       probe0 (17) => trigger_in,
--       probe0 (18) => rst_n, -- std_logic' ('0'), -- fifo_empty,
--       probe0 (19) => std_logic' ('0'), -- fifo_full,
--       probe0 (20) => std_logic' ('0'), -- dma_running,
--       probe0 (21) => std_logic' ('0'), -- dma_start_burst,
--       probe0 (22) => m_axi_dat0_wlast,
--       probe0 (23) => std_logic' ('0'), -- beat_acked,
--       probe0 (31 downto 24) => std_logic_vector'(31 downto 24 => '0'), -- wr_count,
--       probe0 (63 downto 32) => m_axi_dat0_awaddr(31 downto 0));

--  inst_ila: ila_0
--    port map (
--      clk => clk_100m,
--      probe0 (0) => s_awvalid_csr,
--      probe0 (1) => s_awready_csr,
--      probe0 (9 downto 2) => s_awaddr_csr(7 downto 0),
--      probe0 (13 downto 10) => s_wstrb_csr,
--      probe0 (14) => s_wvalid_csr,
--      probe0 (15) => s_wready_csr,
--      probe0 (23 downto 16) => s_wdata_csr(7 downto 0),
--      probe0 (25 downto 24) => s_bresp_csr,
--      probe0 (26) => s_bvalid_csr,
--      probe0 (27) => s_bready_csr,
--      probe0 (28) => s_arvalid_csr,
--      probe0 (29) => s_arready_csr,
--      probe0 (37 downto 30) => s_araddr_csr(7 downto 0),
--      probe0 (38) => s_rvalid_csr,
--      probe0 (39) => s_rready_csr,
--      probe0 (41 downto 40) => s_rresp_csr,
--      probe0 (49 downto 42) => s_rdata_csr(7 downto 0),
--      probe0 (50) => rst_n,
--      probe0 (63 downto 51) => std_logic_vector'(others => '0'));
end arch;
