library ieee;
use ieee.std_logic_1164.all;

library unisim;
use unisim.vcomponents.all;

entity sys_fsi_tx is
  port (
    clk_100_i : std_logic;
    clk_400_i : std_logic;
    rst_n_i : std_logic;

    --  Backplane IOs (lvds)
    bp_io_p_o : out std_logic_vector(1 downto 0);
    bp_io_n_o : out std_logic_vector(1 downto 0);

    dat_i : std_logic_vector(7 downto 0)
  );
end sys_fsi_tx;

architecture arch of sys_fsi_tx is
  type t_data is array (natural range <>) of std_logic_vector (7 downto 0);
  signal din : t_data(1 downto 0);
  signal dout : std_logic_vector(1 downto 0);
begin
  din (0) <= dat_i;
  din (1) <= "10101010";

  gen_pad: for i in din'range generate
    inst_OSERDESE3 : OSERDESE3
      generic map (
        DATA_WIDTH => 8,
        INIT => '0',
        IS_CLKDIV_INVERTED => '0',
        IS_CLK_INVERTED => '0',
        IS_RST_INVERTED => '1',
        SIM_DEVICE => "ULTRASCALE_PLUS"
      )
      port map (
        OQ => dout (i),     --  Note: D0 first, D7 last
        T_OUT => open,
        CLK => clk_400_i,
        CLKDIV => clk_100_i,
        D => din(i),
        RST => rst_n_i,
        T => '0'
      );
    
    inst_obufds: OBUFDS
      port map (
        O =>  bp_io_p_o(i), -- 1-bit output: Diff_p output (connect directly to top-level port)
        OB => bp_io_n_o(i), -- 1-bit output: Diff_n output (connect directly to top-level port)
        I => dout (i)       -- 1-bit input: Buffer input
      );
  end generate;
end arch;