--  Dummy model of ads5295

library ieee;
use ieee.std_logic_1164.all;

entity ads5295 is
  port (
    clk_p_i : std_logic;
    clk_n_i : std_logic;

    --  ADC data
    adc_outa_p_o : out std_logic_vector(7 downto 0);
    adc_outa_n_o : out std_logic_vector(7 downto 0);
    adc_outb_p_o : out std_logic_vector(7 downto 0);
    adc_outb_n_o : out std_logic_vector(7 downto 0);

    --  ADC bit clock
    adc_lclk_p_o : out std_logic;
    adc_lclk_n_o : out std_logic;

    --  ADC frame
    adc_adclk_p_o : out std_logic;
    adc_adclk_n_o : out std_logic;

    --  ADC control
    adc_reset_i : std_logic;
    adc_pd_i    : std_logic;
    adc_sync_i  : std_logic;

    --  ADC SPI
    adc_sclk_i : std_logic;
    adc_sdata_i : std_logic;
    adc_sdout_o : out std_logic;
    adc_cs_n_i : std_logic
  );
end ads5295;

architecture arch of ads5295 is
  signal clk : std_logic;
  signal rst : std_logic;
  signal lclk : std_logic;
  signal adclk : std_logic;

  function image (v : std_logic_vector) return string
  is
    type map_type is array(std_logic) of character;
    constant smap : map_type := "UX01ZWLH-";
    variable res : string (1 to v'length);
    alias av : std_logic_vector(res'range) is v;
  begin
    for i in res'range loop
      res (i) := smap (av (i));
    end loop;
    return res;
  end image;

  subtype t_sample is std_logic_vector (11 downto 0);
  type t_samples_arr is array (7 downto 0) of t_sample;

  signal samples : t_samples_arr := (0 => b"1000_0000_0000",
                                     1 => b"1001_1001_1001",
                                     others => (others => '0'));
begin
  clk <= clk_p_i;
  rst <= adc_reset_i;

  --  SPI rx
  process (clk) is
    variable prev_sclk, prev_cs_n : std_logic;
    variable dat_pos : natural;
    variable rx_dat : std_logic_vector(23 downto 0);
    constant tx_dat : std_logic_vector(23 downto 0) := x"00_de15";
  begin
    if rising_edge(clk) then
      if rst = '1' or adc_cs_n_i = '1' then
        dat_pos := rx_dat'left;
        adc_sdout_o <= '0';
      elsif adc_sclk_i = '1' and prev_sclk = '0' then
        --  Rising edge on sclk
        rx_dat (dat_pos) := adc_sdata_i;
        if dat_pos = 0 then
          report "ads5295: got spi addr=" & image (rx_dat (23 downto 16)) & " dat=" & image (rx_dat (15 downto 0));
          dat_pos := rx_dat'left;
        else
          dat_pos := dat_pos - 1;
        end if;
      elsif prev_cs_n = '1' or (adc_sclk_i = '0' and prev_sclk = '1') then
        --  Falling edge.
        adc_sdout_o <= tx_dat(dat_pos);
      end if;

      prev_sclk := adc_sclk_i;
      prev_cs_n := adc_cs_n_i;
    end if;
  end process;

  adclk <= clk after 1 ns;

  adc_adclk_p_o <= adclk;
  adc_adclk_n_o <= not adclk;

  adc_lclk_p_o <= lclk;
  adc_lclk_n_o <= not lclk;

  --  Serialization of data
  process is
    variable b : std_logic;
  begin
    lclk <= '0';
    loop
      wait until rising_edge(adclk);
      for i in 5 downto 0 loop
        for ch in 0 to 7 loop
          b := samples (ch)(i + 6);
          adc_outb_p_o (ch) <= b;
          adc_outb_n_o (ch) <= not b;
    
          b := samples (ch)(i);
          adc_outa_p_o (ch) <= b;
          adc_outa_n_o (ch) <= not b;
        end loop;
        wait for 1000 ps;
        if i = 5 then
          lclk <= not lclk;
        end if;
        wait for 666 ps;
      end loop;
    end loop;
  end process;
end;
