library ieee;
use ieee.std_logic_1164.all;

entity prbs_tb is
  generic (NBR : natural := 20);
end prbs_tb;

architecture arch of prbs_tb is
  signal clk, rst_n : std_logic;
  signal dato : std_logic_vector(7 downto 0);
  signal err : std_logic_vector(7 downto 0);

  function to_string(v : std_logic_vector) return string is
    variable res: string (1 to v'length);
    alias va : std_logic_vector(res'range) is v;
  begin
    for i in res'range loop
      case va(i) is
        when '1' | 'H' => res (i) := '1';
        when '0' | 'L' => res (i) := '0';
        when others => res (i) := '?';
      end case;
    end loop;
    return res;
  end to_string;
begin
  inst_prbs: entity work.fsi_prbs_gen
    generic map (
      init => x"01"
    )
  port map (
    clk_i => clk,
    rst_n_i => rst_n,
    out_o => dato
  );

  my_inst: entity work.sys_prbs_chk
    port map (
      clk_i => clk,
      rst_n_i => rst_n,
      dat_i => dato,
      err_o => err
    );
  process
    procedure pulse is
    begin
      clk <= '0';
      wait for 1 ns;
      clk <= '1';
      wait for 1 ns;
    end pulse;
  begin
    rst_n <= '0';
    pulse;
    rst_n <= '1';

    for i in 1 to NBR loop
      report "prbs: " & to_string(dato) & ", err: " & to_string(err) severity note;
      pulse;
    end loop;
    wait;
  end process;
end arch;